#include "/lib/noise.glsl"

// [0] is top, [1] is ground
vec3[2] calc_sky_coef() {
    vec3[2] FinalSky;
    const vec3 SKY_TOP_NOON = vec3(NOON_SKY_T_R, NOON_SKY_T_G, NOON_SKY_T_B);
    const vec3 SKY_TOP_SUNRISE = vec3(SUNRISE_SKY_T_R, SUNRISE_SKY_T_G, SUNRISE_SKY_T_B);
    const vec3 SKY_TOP_SUNSET = vec3(SUNSET_SKY_T_R, SUNSET_SKY_T_G, SUNSET_SKY_T_B);
    const vec3 SKY_TOP_NIGHT = vec3(NIGHT_SKY_T_R, NIGHT_SKY_T_G, NIGHT_SKY_T_B);

    const vec3 SKY_GROUND_NOON = vec3(NOON_SKY_G_R, NOON_SKY_G_G, NOON_SKY_G_B);
    const vec3 SKY_GROUND_SUNRISE = vec3(SUNRISE_SKY_G_R, SUNRISE_SKY_G_G, SUNRISE_SKY_G_B);
    const vec3 SKY_GROUND_SUNSET = vec3(SUNSET_SKY_G_R, SUNSET_SKY_G_G, SUNSET_SKY_G_B);
    const vec3 SKY_GROUND_NIGHT = vec3(NIGHT_SKY_G_R, NIGHT_SKY_G_G, NIGHT_SKY_G_B);

    FinalSky[0] = mix(mix(mix(SKY_TOP_SUNSET, SKY_TOP_NIGHT, nightStrength), SKY_TOP_NOON, dayStrength), SKY_TOP_SUNRISE, sunriseStrength);
    FinalSky[1] = mix(mix(mix(SKY_GROUND_SUNSET, SKY_GROUND_NIGHT, nightStrength), SKY_GROUND_NOON, dayStrength), SKY_GROUND_SUNRISE, sunriseStrength);

    FinalSky[0] = apply_saturation(FinalSky[0], 1 - rainStrength) * (1 - rainStrength*0.25);
    FinalSky[1] = apply_saturation(FinalSky[1], 1 - rainStrength/2)* (1 - rainStrength*0.33);
    return FinalSky;
} 

float fogify(float x, float w) {
	return w / (x * x + w);
}

vec3 get_clouds(vec3 CloudPos) {
    #if CLOUD_SPEED != 0
        #ifndef DIMENSION_END
        const int ACTUAL_CLOUD_SPEED = 75-CLOUD_SPEED*5;
        float Animation = frameTimeCounter/ACTUAL_CLOUD_SPEED;
        const int ACTUAL_CLOUD_SIZE = CLOUD_SIZE*100;
        #else
        const int ACTUAL_CLOUD_SPEED = 20-CLOUD_SPEED*2;
        float Animation = frameTimeCounter/ACTUAL_CLOUD_SPEED;
        #endif
    #else
        float Animation = 0;
    #endif
	
    float CloudLen = length(CloudPos.xz/CloudPos.y);
    
    #ifndef DIMENSION_END
    float Noise = fbm(CloudPos.xz+Animation, CLOUD_QUALITY);
    Noise = pow(Noise, CLOUD_AMOUNT - rainStrength*5)*ACTUAL_CLOUD_SIZE;

    Noise = ACESFilm(vec3(Noise)).r; // Yes, I'm tonemapping the clouds. Cry about it.
    Noise *= max(1 - CloudLen/24, 0); // Reduce cloud amount further away
    vec3 SkyCol = apply_saturation(calc_sky_coef()[1], 1.2);
	return dither(Noise * SkyCol * (0.7-rainStrength/4), CloudPos.xz);
    #else
    float Noise = fbm(vec2(CloudPos.x-Animation, CloudPos.z+Animation), CLOUD_QUALITY);
    Noise = pow(Noise, 14)*500;

    float BlackHole = max(8-CloudLen, 0);
    if(BlackHole < 0.5) BlackHole *= -BlackHole;
    Noise *= 1 - CloudLen/16;

    return dither((max(Noise, 0)-BlackHole) * vec3(40, 15, 2), CloudPos.xz);
    #endif
}

vec3 calcSkyColor(vec3 pos, vec3 SunVec) {
	float upDot = dot(pos, gbufferModelView[1].xyz)+0.1; //not much, what's up with you?
    vec3[2] FinalSky = calc_sky_coef();

    // Sun glare
    float Dist = clamp(dot(pos, SunVec), 0, 1);
    Dist *= Dist;
    float Visibility = cos(timeAngle*2*PI); 
    Visibility *= Visibility;
    Visibility *= Visibility;
    Visibility *= Visibility;
    Visibility *= Visibility;
    
    const vec3 SUN_GLARE = vec3(0.7, 0.25, 0.0);
    FinalSky[1] += SUN_GLARE*Dist*Visibility;

	return dither(mix(FinalSky[0], FinalSky[1], fogify(max(upDot, 0.0), 0.05)), pos.xy);
}

float get_stars(vec3 PlayerPos) {
    vec3 StarCoord = PlayerPos / (PlayerPos.y + length(PlayerPos.xz));
	const float ACTUAL_STAR_SIZE = STAR_SIZE * 512;
	StarCoord = floor(StarCoord * ACTUAL_STAR_SIZE) / ACTUAL_STAR_SIZE;
	return pow(random(StarCoord.xz), 1024) * clamp(sqrt(StarCoord.y+0.1), 0, 1) * nightStrength * STAR_STRENGTH;
}