#include "/lib/all_the_uniforms.glsl"
varying vec2 texcoord;
#include "/lib/util.glsl"
#include "/lib/tonemap.glsl"
#include "/lib/blur.glsl"
#include "/lib/settings.glsl"
#include "/lib/taa.glsl"

// Color grading, Bloom final, TAA final

/* DRAWBUFFERS:03 */

void main() {
	vec4 Color = texture2D(colortex0, texcoord);
	#if TAA_MODE != 0
	gl_FragData[1] = TAA(Color.xyz, texture2D(colortex3, texcoord).a);
	#endif
	#ifdef BLOOM 
		float Offset = 0;
		vec4 FinalBloom = vec4(0, 0, 0, 1);
		for(int i = 2; i <= 2+BLOOM_SIZE; i++) {
			FinalBloom += texture2D(colortex1, texcoord/pow(2, i)+Offset);
			Offset += 1/pow(2, i);
		}
		Color += FinalBloom * BLOOM_STRENGTH;
	#endif

	Color.xyz = ACESFilm(Color.xyz);

	Color.xyz = apply_vibrance(Color.xyz, VIBRANCE);
	Color.xyz = apply_saturation(Color.xyz, SATURATION);

	vec3 MinBright = vec3(TONEMAP_MIN_R, TONEMAP_MIN_G, TONEMAP_MIN_B);
	#ifdef FILM_GRAIN
	MinBright += random(gl_FragCoord.xy/64+random(vec2(frameTimeCounter)))*0.05;
	#endif
	Color.xyz = max(Color.xyz, MinBright);

	Color.xyz = apply_contrast(Color.xyz, CONTRAST);

	gl_FragData[0] = Color;
	#ifdef DEBUG_BUFFER
	Color = texture2D(colortex1, texcoord);
	#endif
}
