#include "/lib/all_the_uniforms.glsl"

#include "/lib/settings.glsl"
#include "/lib/tonemap.glsl"

#include "/lib/util.glsl"
#include "/global/sky.glsl"
#include "/global/fog.glsl"

// Sky, Clouds, etc

varying vec2 texcoord;
varying vec3 SunVec;

/* DRAWBUFFERS:01 */

void main() {
	gl_FragData[0] = texture2D(colortex0, texcoord);
	float Depth = texture2D(depthtex0, texcoord).x;

	vec3 ViewPos = to_view_pos(vec3(texcoord, Depth));
	vec3 PlayerPos = mat3(gbufferModelViewInverse) * ViewPos;

	#ifdef DIMENSION_OVERWORLD
	vec3 SkyColor = calcSkyColor(normalize(ViewPos), SunVec);
	#elif defined DIMENSION_END
	vec3 SkyColor = fogColor*1.5;
	#else
	float LightenFactor = 1.5 - get_luminance(fogColor.xyz); // Makes all biomes roughly equally bright
	LightenFactor *= LightenFactor;
	vec3 SkyColor = apply_saturation(fogColor, 0.7)*LightenFactor;
	#endif
	gl_FragData[1].rgb = SkyColor;

	if(Depth >= 1) {
		#ifdef DIMENSION_NETHER
		gl_FragData[0] = vec4(0, 0, 0, 1);
		#endif
		gl_FragData[0].xyz += SkyColor;	
		if(PlayerPos.y > 0) {
			#ifdef DIMENSION_OVERWORLD
			gl_FragData[0].xyz += get_stars(PlayerPos);
			#endif

			#if defined FANCY_CLOUDS && !defined DIMENSION_NETHER
				vec3 CloudPos = normalize(PlayerPos); 
				#ifdef DIMENSION_END
				// Adding 5 here makes it look like a black hole
				CloudPos = vec3((CloudPos.xz + 5) / CloudPos.y, 1).xzy;
				#else
				CloudPos = vec3((CloudPos.xz) / CloudPos.y, 1).xzy;
				#endif
				gl_FragData[0].xyz += get_clouds(CloudPos);
			#endif
		}
		
	}
}

